{% extends "base.html" %}
{% load crispy_forms_tags %}

{% block content %}
<h1 class="text-center"> {{ header }} </h1>
<hr>
<div class="col-flex" style="max-width:500px">
    <form method="POST">{% csrf_token %}
        <fieldset class='form-group'>
            <legend class="border-bottom mb-4">Login</legend>
            {{ form|crispy }}
        </fieldset>
        <div class="form-group">
            <button type="btn btn-outline-info" name="submit">Login</button>
        </div>
    </form>
</div>
{% endblock content %}