{% extends extend_str %}
{% load aux_tags %}
{% load static %}

{% block content %}

<form method="post">{% csrf_token %}
    <p>
    {% if message %}
        <h4>{{ message }}</h4>
    {% else %} 
        <h4>Are you sure you want to delete this "{{ object|render_class_name }}" instance?</h4>
    {% endif %}
    {% if extra_message %}
        <h6>{{ extra_message }}</h6>
    {% endif %}
    <br>
    <h5><b> {{ object }}</b></h5>
    <br>
    <b><font color='red'>This cannot be undone.</font></b>
    </p>
    <input class="btn btn-primary" name="submit" type="submit" value="Confirm">

    <input class="btn btn-warning" type="button" , value="Cancel" onclick="{{ onclick_cancel_action|null_default:'history.back()' }}">


</form>

{% endblock content %}


{% block extra_javascript %}
    {% for ejs in extra_js %}
        <script src="{% static ejs %}"></script>
    {% endfor %}
{% endblock %}