{% extends extend_str %}
{% load plotly_dash %}
{% load crispy_forms_tags %}
{% block title %}{% endblock %}
{% load static %}

{% block content %}
    <h1> {{ disclaimer }} </h1>
    {% if link_name %}
    <a href="{% url link_name %}">{{ link_text }}</a>
    {% endif %}

    <div class='text-center'>
        <h2> {{ header }} </h2>
    </div>
    <h2 class="text-center"> {{ subheader }} </h2>

    {% if filter %}
        <h3 class="text-center"> {{ filter_header }} </h3>
        <h4> Filter Results </h4>
        <form action="" method="get" enctype="multipart/form-data">
            {% crispy filter.form %}
        </form>
    {% endif %}


    <div style={{ style_str }}>
        {% if initial_arguments %}
            {% plotly_app_bootstrap name=plotly_app_name initial_arguments=initial_arguments %}
        {% else %}
            {% plotly_app_bootstrap name=plotly_app_name %}
        {% endif %}
    </div>
    
{% endblock %}


{% block extra_javascript %}
    {% for ejs in extra_js %}
        <script src="{% static ejs %}"></script>
    {% endfor %}
{% endblock %}