# Generated by Django 4.0.6 on 2023-03-03 14:03

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Country',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('alpha2', models.CharField(max_length=2, null=True)),
                ('alpha3', models.CharField(max_length=3, unique=True)),
                ('num', models.PositiveIntegerField(unique=True)),
                ('name', models.CharField(max_length=250)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Subdivision',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('iso_code', models.CharField(max_length=10, unique=True)),
                ('name', models.CharField(max_length=250)),
                ('category', models.CharField(max_length=50)),
                ('country', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='django_aux_geo.country')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
